// -------------------------------------------------------------- -*- C++ -*-
// File: iloCPXCALLBACKSOLUTIONSTRATEGYenum.h
// Version 12.10.0
// --------------------------------------------------------------------------
// Licensed Materials - Property of IBM
// 5725-A06 5725-A29 5724-Y48 5724-Y49 5724-Y54 5724-Y55 5655-Y21
// Copyright IBM Corporation 2000, 2019. All Rights Reserved.
// 
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
// --------------------------------------------------------------------------
NoCheck = CPXCALLBACKSOLUTION_NOCHECK,
CheckFeasible = CPXCALLBACKSOLUTION_CHECKFEAS,
Propagate = CPXCALLBACKSOLUTION_PROPAGATE,
Solve = CPXCALLBACKSOLUTION_SOLVE
