// -------------------------------------------------------------- -*- C++ -*-
// File: iloCPXCALLBACKINFOenum.h
// Version 12.10.0
// --------------------------------------------------------------------------
// Licensed Materials - Property of IBM
// 5725-A06 5725-A29 5724-Y48 5724-Y49 5724-Y54 5724-Y55 5655-Y21
// Copyright IBM Corporation 2000, 2019. All Rights Reserved.
// 
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
// --------------------------------------------------------------------------
ThreadId = CPXCALLBACKINFO_THREADID,
NodeCount = CPXCALLBACKINFO_NODECOUNT,
IterationCount = CPXCALLBACKINFO_ITCOUNT,
BestSolution = CPXCALLBACKINFO_BEST_SOL,
BestBound = CPXCALLBACKINFO_BEST_BND,
Threads = CPXCALLBACKINFO_THREADS,
Feasible = CPXCALLBACKINFO_FEASIBLE,
Time = CPXCALLBACKINFO_TIME,
DeterministicTime = CPXCALLBACKINFO_DETTIME,
NodeUID = CPXCALLBACKINFO_NODEUID,
NodeDepth = CPXCALLBACKINFO_NODEDEPTH,
CandidateSource = CPXCALLBACKINFO_CANDIDATE_SOURCE,
Restarts = CPXCALLBACKINFO_RESTARTS
